#include "control.h"

#include <iostream.h>
#include <GL/glut.h>
#include <math.h>
#include "lights.h"
#include "main.h"
#include "robot.h"
#include "scene.h"
#include "camera.h"


enum {
        M_QUIT,
        M_ADJUST_AMBIENT,

	M_ADJUST_LIGHT0_POS,
	M_ADJUST_LIGHT0_DIF,
	M_ADJUST_LIGHT0_SPEC,

	M_ADJUST_LIGHT2_POS,
	M_ADJUST_LIGHT2_DIR,
	M_ADJUST_LIGHT2_CUT,
	M_ADJUST_LIGHT2_DIF,
	M_ADJUST_LIGHT2_SPEC,

	M_ADJUST_LIGHT3_POS,
	M_ADJUST_LIGHT3_DIR,
	M_ADJUST_LIGHT3_CUT,
	M_ADJUST_LIGHT3_DIF,
	M_ADJUST_LIGHT3_SPEC,

	M_ADJUST_LIGHT4_POS,
	M_ADJUST_LIGHT4_DIF,
	M_ADJUST_LIGHT4_SPEC,

        M_HELP
} MENU_ITEMS;


int mouse_state  = -1;
int mouse_button = -1;
int old_x = -1;
int old_y = -1;

bool verbose = true;

float time     = 0.0;
float old_time = 0.0;
float elapsed  = 0.0;
float gravity  = -0.1;


int make_menu ()
{
  int light0_menu = glutCreateMenu(menu_func);
  glutAddMenuEntry("Position", M_ADJUST_LIGHT0_POS);
  glutAddMenuEntry("Diffuse",  M_ADJUST_LIGHT0_DIF);
  glutAddMenuEntry("Specular", M_ADJUST_LIGHT0_SPEC);

  int light2_menu = glutCreateMenu(menu_func);
  glutAddMenuEntry("Position",  M_ADJUST_LIGHT2_POS);
  glutAddMenuEntry("Direction", M_ADJUST_LIGHT2_DIR);
  glutAddMenuEntry("Cutoff",    M_ADJUST_LIGHT2_CUT);
  glutAddMenuEntry("Diffuse",   M_ADJUST_LIGHT2_DIF);
  glutAddMenuEntry("Specular",  M_ADJUST_LIGHT2_SPEC);

  int light3_menu = glutCreateMenu(menu_func);
  glutAddMenuEntry("Position",  M_ADJUST_LIGHT3_POS);
  glutAddMenuEntry("Direction", M_ADJUST_LIGHT3_DIR);
  glutAddMenuEntry("Cutoff",    M_ADJUST_LIGHT3_CUT);
  glutAddMenuEntry("Diffuse",   M_ADJUST_LIGHT3_DIF);
  glutAddMenuEntry("Specular",  M_ADJUST_LIGHT3_SPEC);

  int light4_menu = glutCreateMenu(menu_func);
  glutAddMenuEntry("Position", M_ADJUST_LIGHT4_POS);
  glutAddMenuEntry("Diffuse",  M_ADJUST_LIGHT4_DIF);
  glutAddMenuEntry("Specular", M_ADJUST_LIGHT4_SPEC);

  int main = glutCreateMenu(menu_func);
  glutAddMenuEntry("Adjust Ambient", M_ADJUST_AMBIENT);
  glutAddSubMenu("Adjust Light0",  light0_menu);
  glutAddSubMenu("Adjust Light2",  light2_menu);
  glutAddSubMenu("Adjust Light3",  light3_menu);
  glutAddSubMenu("Adjust Light4",  light4_menu);
  glutAddMenuEntry("Help", M_HELP);
  glutAddMenuEntry("Quit", M_QUIT);

  glutAttachMenu(GLUT_RIGHT_BUTTON);

  return main;
}



void menu_func (int value)
{
  switch (value)
  {
  case M_QUIT:  
      exit(0);
      break;
      ;;

  case M_ADJUST_AMBIENT:
      menu_adjust_light_prop(ambient, L_AMBIENT);
      break;
      ;;

  case M_ADJUST_LIGHT0_POS:
      menu_adjust_light_prop(light0[2], L_POSITION);
      break;
      ;;
  case M_ADJUST_LIGHT0_DIF:
      menu_adjust_light_prop(light0[0], L_DIFFUSE);
      break;
      ;;
  case M_ADJUST_LIGHT0_SPEC:
      menu_adjust_light_prop(light0[1], L_SPECULAR);
      break;
      ;;

  case M_ADJUST_LIGHT2_POS:
      menu_adjust_light_prop(light2[2], L_POSITION);
      break;
      ;;
  case M_ADJUST_LIGHT2_DIR:
      menu_adjust_light_prop(light2[4], L_DIRECTION);
      break;
      ;;
  case M_ADJUST_LIGHT2_CUT:
      menu_adjust_light_prop(&light2cutoff, L_CUTOFF);
      break;
      ;;
  case M_ADJUST_LIGHT2_DIF:
      menu_adjust_light_prop(light2[0], L_DIFFUSE);
      break;
      ;;
  case M_ADJUST_LIGHT2_SPEC:
      menu_adjust_light_prop(light2[1], L_SPECULAR);
      break;
      ;;

  case M_ADJUST_LIGHT3_POS:
      menu_adjust_light_prop(light3[2], L_POSITION);
      break;
      ;;
  case M_ADJUST_LIGHT3_DIR:
      menu_adjust_light_prop(light3[4], L_DIRECTION);
      break;
      ;;
  case M_ADJUST_LIGHT3_CUT:
      menu_adjust_light_prop(&light3cutoff, L_CUTOFF);
      break;
      ;;
  case M_ADJUST_LIGHT3_DIF:
      menu_adjust_light_prop(light3[0], L_DIFFUSE);
      break;
      ;;
  case M_ADJUST_LIGHT3_SPEC:
      menu_adjust_light_prop(light3[1], L_SPECULAR);
      break;
      ;;

  case M_ADJUST_LIGHT4_POS:
      menu_adjust_light_prop(light4[2], L_POSITION);
      break;
      ;;
  case M_ADJUST_LIGHT4_DIF:
      menu_adjust_light_prop(light4[0], L_DIFFUSE);
      break;
      ;;
  case M_ADJUST_LIGHT4_SPEC:
      menu_adjust_light_prop(light4[1], L_SPECULAR);
      break;
      ;;

  case M_HELP:
      menu_help();
      break;
      ;;

  default:
      break;
      ;;
  }
}


void keyboard_func (unsigned char key, int x, int y)
{
  switch (key)
  {
  case 'r':
      key_rotate_shoulder_theta(3);
      break;
  case 'f':
      key_rotate_shoulder_theta(-3);
      break;

  case 't':
      key_rotate_shoulder_pitch(3);
      break;
  case 'g':
      key_rotate_shoulder_pitch(-3);
      break;

  case 'e':
      key_rotate_elbow(3);
      break;
  case 'd':
      key_rotate_elbow(-3);
      break;

  case 'w':
      key_rotate_wrist(3);
      break;
  case 's':
      key_rotate_wrist(-3);
      break;

  case 'j':
      key_rotate_head_theta(3);
      break;
  case 'l':
      key_rotate_head_theta(-3);
      break;

  case 'i':
      key_rotate_head_pitch(3);
      break;
  case 'k':
      key_rotate_head_pitch(-3);
      break;

  case 'v':
      key_viewpoint_toggle();
      break;

  case ' ':
      key_jump();
      break;

  default:
      cerr << "k " << (int) key << endl;;
      break;
      ;;
  }
}


void special_func (int key, int x, int y)
{
  switch (key)
  {
  case GLUT_KEY_LEFT:
      key_rotate_robot(3);
      break;
      ;;

  case GLUT_KEY_RIGHT:
      key_rotate_robot(-3);
      break;
      ;;

  case GLUT_KEY_UP:
      key_move_robot(1);
      break;
      ;;

  case GLUT_KEY_DOWN:
      key_move_robot(-1);
      break;
      ;;

  default:
      cerr << "s " << (int) key << endl;;
      break;
      ;;
  }
}
  
        
void mouse_click_func (int button, int state, int x, int y)
{
  mouse_state  = state;
  mouse_button = button;
  old_x = x;
  old_y = y;
}
        
  
void mouse_move_func (int x, int y)
{
  if (mouse_button == GLUT_LEFT_BUTTON)
  {
      if (camera_mode == C_THIRD)
      {
          mouse_move_camera(x - old_x, y - old_y);
      }
      else if (camera_mode == C_FIRST)
      {
          key_rotate_head_theta(old_x - x);
          key_rotate_head_pitch(old_y - y);
      }
  }
  else if (mouse_button == GLUT_MIDDLE_BUTTON)
  {
      if (camera_mode == C_THIRD)
      {
          mouse_zoom_camera(x - old_x, y - old_y);
      }
      else if (camera_mode == C_FIRST)
      {
          key_rotate_robot(old_x - x);
          key_move_robot(old_y - y);
      }
  }

  old_x = x;
  old_y = y;
}


void idle_func ()
{
  time     = glutGet(GLUT_ELAPSED_TIME) / 1000.0;
  elapsed  = time - old_time;

  robot_body_z += robot_body_vel_z * elapsed * 20;
  robot_body_vel_z += gravity * elapsed * 20;

  if (robot_body_z <= 2)
  {
      robot_body_z = 2;
      robot_body_vel_z = 0;
  }

  glutPostRedisplay();
  old_time = time;
}


void visibility_func (int visibility)
{
  if (visibility == GLUT_VISIBLE)
      glutIdleFunc(idle_func);
  else
      glutIdleFunc(NULL);
}


void menu_adjust_light_prop (float* v, int type)
{
  float a, b, c;

  switch (type)
  {
  case L_POSITION:
      cerr << "New position:  (old = " << v[0] << "," << v[1] 
           << "," << v[2] << ")" << endl;
      cin >> a >> b >> c;

      v[0] = a;
      v[1] = b;
      v[2] = c;
      break;
      ;;

  case L_DIRECTION:
      cerr << "New direction:  (old = " << v[0] << "," << v[1] 
           << "," << v[2] << ")" << endl;
      cin >> a >> b >> c;

      v[0] = a;
      v[1] = b;
      v[2] = c;
      break;
      ;;

  case L_CUTOFF:
      cerr << "New cutoff [0,360]:  (old = " << v[0] << ")" << endl;
      cin >> a;

      if (a < 0 || a > 360)
      {
          cerr << "Invalid value" << endl;
          return;
      }

      v[0] = a;
      break;
      ;;

  case L_DIFFUSE:
      cerr << "New diffuse [0,1]:  (old = " << v[0] << "," << v[1] 
           << "," << v[2] << ")" << endl;
      cin >> a >> b >> c;

      if (a < 0 || a > 1 || b < 0 || b > 1 || c < 0 || c > 1)
      {
          cerr << "Invalid input" << endl;
          return;
      }

      v[0] = a;
      v[1] = b;
      v[2] = c;
      break;
      ;;

  case L_SPECULAR:
      cerr << "New specular [0,1]:  (old = " << v[0] << "," << v[1] 
           << "," << v[2] << ")" << endl;
      cin >> a >> b >> c;

      if (a < 0 || a > 1 || b < 0 || b > 1 || c < 0 || c > 1)
      {
          cerr << "Invalid input" << endl;
          return;
      }

      v[0] = a;
      v[1] = b;
      v[2] = c;
      break;
      ;;

  case L_AMBIENT:
      cerr << "New ambient [0,1]:  (old = " << v[0] << "," << v[1] 
           << "," << v[2] << ")" << endl;
      cin >> a >> b >> c;

      if (a < 0 || a > 1 || b < 0 || b > 1 || c < 0 || c > 1)
      {
          cerr << "Invalid input" << endl;
          return;
      }

      v[0] = a;
      v[1] = b;
      v[2] = c;
      break;
      ;;

  }

  glutPostRedisplay();
}


void menu_help ()
{
  cerr << endl
       << "key\t\t\taction" << endl 
       << endl
       << "Left / Right\t\tTurn body left / right" << endl
       << "Up / Down\t\tMove forward / backward" << endl
       << "J / L\t\t\tTurn head left / right" << endl
       << "I / K\t\t\tNod head up / down" << endl
       << "V\t\t\tToggle first / third person viewpoint" << endl
       << endl
       << "T / G\t\t\tRotate shoulder in / out" << endl
       << "R / F\t\t\tRotate shoulder up / down" << endl
       << "E / D\t\t\tRotate elbow up / down" << endl
       << "W / S\t\t\tRotate wrist right / left" << endl
       << endl
       << "<SPACE>\t\t\tJump" << endl
       << endl
       << "mouse button\t\taction\t\tThird Person" << endl
       << endl
       << "left\t\t\tRotate camera" << endl
       << "middle\t\t\tRaise / lower camera" << endl
       << "right\t\t\tPop-up menu" << endl
       << endl
       << "mouse button\t\taction\t\tFirst Person" << endl
       << endl
       << "left\t\t\tTurn / nod head" << endl
       << "middle\t\t\tMove / turn body" << endl
       << "right\t\t\tPop-up menu" << endl
       << endl;
}


void mouse_move_camera (int dx, int dy)
{
  camera_theta += dx * 0.5;
  camera_pitch += dy * 0.5;

  glutPostRedisplay();
}


void mouse_zoom_camera (int dx, int dy)
{
  camera_zoom += dy * 0.01;

  glutPostRedisplay();
}


void key_move_robot (int s)
{
  robot_body_x  += s * cos(deg2rad(robot_body_theta)) * 0.5;
  robot_body_y  += s * sin(deg2rad(robot_body_theta)) * 0.5;

  if (verbose)
      cerr << "robot_body_position: " << robot_body_x << "," 
           << robot_body_y << endl;

  glutPostRedisplay();
}


void key_rotate_robot (int s)
{
  robot_body_theta += s;

  if (verbose)
      cerr << "robot_body_theta: " << robot_body_theta << endl;

  glutPostRedisplay();
}


void key_rotate_shoulder_theta (int s)
{
  robot_shoulder_theta += s;

  if (verbose)
      cerr << "robot_shoulder_theta: " << robot_shoulder_theta << endl;

  glutPostRedisplay();
}


void key_rotate_shoulder_pitch (int s)
{
  robot_shoulder_pitch += s;

  if (verbose)
      cerr << "robot_shoulder_pitch: " << robot_shoulder_pitch << endl;

  glutPostRedisplay();
}


void key_rotate_elbow (int s)
{
  robot_elbow_theta += s;

  if (verbose)
      cerr << "robot_elbow_theta: " << robot_elbow_theta << endl;

  glutPostRedisplay();
}


void key_rotate_wrist (int s)
{
  robot_wrist_theta += s;

  if (verbose)
      cerr << "robot_wrist_theta: " << robot_wrist_theta << endl;

  glutPostRedisplay();
}


void key_rotate_head_theta (int s)
{
  robot_head_theta += s;

  if (verbose)
      cerr << "robot_head_theta: " << robot_head_theta << endl;

  glutPostRedisplay();
}


void key_rotate_head_pitch (int s)
{
  robot_head_pitch += s;

  if (verbose)
      cerr << "robot_head_pitch: " << robot_head_pitch << endl;

  glutPostRedisplay();
}


void key_viewpoint_toggle ()
{
  if (camera_mode == C_THIRD)
      camera_mode = C_FIRST;
  else
      camera_mode = C_THIRD;

  glutPostRedisplay();
}


void key_jump ()
{
  robot_body_vel_z = 1.0;
}
