#include "main.h"

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <GL/glut.h>
#include "control.h"
#include "lights.h"
#include "robot.h"
#include "scene.h"
#include "camera.h"
#include "bmp.h"


int  window_width  = 300;
int  window_height = 300;

bool write_frames  = false;
int  frame_num     = 0;
char dir[256]      = "/tmp/sdiverdi/movie";
char file_name[256];
Image* bmp;
unsigned char* frame;


int main (int argc, char** argv)
{
  // set up the window
  glutInit(&argc, &argv[0]); 
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
  glutInitWindowPosition(100,100);
  glutInitWindowSize(window_width, window_height);
  glutCreateWindow("robot");
 
  // initialize parameters
  init();

  // register call back functions
  glutDisplayFunc(display);
  glutReshapeFunc(reshape);

  // wait for something to happen
  glutMainLoop();

  return 0;
}


void init()
{
  glClearColor(0.0,0.0,0.0,0.0);
  glShadeModel(GL_SMOOTH);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_CULL_FACE);
  glEnable(GL_NORMALIZE);

  // main menu
  make_menu();

  // callbacks
  glutKeyboardFunc(keyboard_func);
  glutMouseFunc(mouse_click_func);
  glutMotionFunc(mouse_move_func);
  glutSpecialFunc(special_func);
  glutIdleFunc(idle_func);
  glutVisibilityFunc(visibility_func);

  glMap2f(GL_MAP2_VERTEX_3, 0, 1, 3, 4, 0, 1, 12, 4, &mesh_points[0][0][0]);
  glEnable(GL_MAP2_VERTEX_3);
  glEnable(GL_AUTO_NORMAL);
  glMapGrid2f(20, 0.0, 1.0, 20, 0.0, 1.0);

  calc_shadow_matrix(floor_shadow0, floor_plane, light0[2]);
  calc_shadow_matrix(floor_shadow2, floor_plane, light2[2]);
  calc_shadow_matrix(floor_shadow3, floor_plane, light3[2]);
  calc_shadow_matrix(floor_shadow4, floor_plane, light4[2]);

  initSceneDisplayLists();
  initTexture();

  if (write_frames)
  {
      frame = new unsigned char[window_width*window_height*3];
      bmp   = ImageNew(window_width, window_height);
  }
}


void display(void) 
{
  // check if there have been any openGL problems
  GLenum errCode = glGetError();
  if (errCode != GL_NO_ERROR) 
  {
      const GLubyte* errString = gluErrorString(errCode);
      cerr << "OpenGL error: " << errString << endl;
  }

  // clear the frame buffer
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

  position_camera();
  make_lights();

  // draw the scene
  drawEverything();

  // write the frame to a file
  if (write_frames)
  {
      glPixelStorei(GL_PACK_ALIGNMENT, 1);
      glReadPixels(0, 0, window_width, window_height, 
                   GL_RGB, GL_UNSIGNED_BYTE, frame);

      ImageSetPixels(bmp, frame);

      sprintf(file_name, "%s/file%03d.bmp", dir, frame_num);
      FILE* file = fopen(file_name, "wb");
      BMPWriteImage(bmp, file);
      fclose(file);
  }

  // swap buffers
  glutSwapBuffers();  

  frame_num++;
}


void reshape (int width, int height) 
{
  // set window height and width
  window_width  = width;
  window_height = height; 

  // the lower left corner of the viewport is 0,0
  // the upper right corner is width, height
  glViewport(0, 0, (GLint) width, (GLint) height);  

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(frustum[0], frustum[1], frustum[2], frustum[3],
            frustum[4], frustum[5]);
  glPushMatrix();

  // default mode should be modelview
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}
